/****************************************************************************
   File:        ProcessorImpl.java
   Description: Implementation of a Processor. This will be the agent responsible for receiving processes and executing them.
   Project:     MAC 322 - Distributed Scheduler
   Authors:     Nelio Pereira (nelio@ime.usp.br)
   Institution: Department of Computer Science
                University of Sao Paulo
******************************************************************************/
     
package sod.client;

import java.net.*;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.rmi.server.UnicastRemoteObject;
import java.util.*;

import sod.util.Process;
import sod.util.Processor;
import sod.util.Registry;


public class ProcessorImpl extends UnicastRemoteObject implements Processor { 

    private Hashtable processes;

    private String hostAddress;
    private String hostName;
    private String serverName;
    
    public ProcessorImpl(String serverName) throws RemoteException { 
	super();
	processes = new Hashtable();
	try {
	    hostAddress = InetAddress.getLocalHost().getHostAddress();
	    hostName = InetAddress.getLocalHost().getHostName();
	} catch (UnknownHostException e) {
	    e.printStackTrace();
	}
	this.serverName = serverName;
    } 

    public String getHostName() {
	return hostName;
    }

    public String getHostAddress() {
	return hostAddress;
    }

    public String getServerName() {
        return serverName;
    }

    public void execute(Process process) throws RemoteException {
	System.out.println("Executing " + process.getClassName());
	processes.put(new Integer(process.getPID()), process);
	if (process.isStarted()) {
	    process.resumeExecution();
	}
	process.start();
    }
    
    public int getScore() throws RemoteException {
	return getNumProcesses();
    }
    
    public int getNumProcesses() throws RemoteException {
	int ret = 0;
	Vector procs = getProcesses();
	for (int i=0; i < procs.size(); i++) {
	    Process process = (Process)procs.get(i);
	    if (process.isAlive()) {
		ret++;
	    }
	}
	return ret;
    }
    
    public void exit(long delay) throws RemoteException {
	KillTask killTask = new KillTask();
	System.out.println("Going down in " + delay + " milliseconds.");
	new Timer().schedule(killTask, delay);
    }

    public Vector getProcesses() throws RemoteException {
	return new Vector(processes.values());
    }

    /**
     *
     * For simplicity, returns only the pids of numProcesses processes.
     */
    public int[] getPIDs(int numProcesses) throws RemoteException {
	Vector ret = getProcesses();
	int[] pids = new int[numProcesses];
	for (int i=0; i < numProcesses; i++) {
	    pids[i] = ((Process)ret.get(i)).getPID();
	}
	return pids;
    }

    public void delProcess(int pid) throws RemoteException {
	processes.remove(new Integer(pid));
    } 
    
    public Process suspendProcess(int pid) throws RemoteException {
	Process process = (Process)processes.get(new Integer(pid));
	process.stopExecution();
	System.out.println("Suspending " + process.getClassName());
	return process;
    }
    
    
    /* * * * * * * * * * * * * * * * * * * */


    private static void usage() {
	System.out.println("java ProcessorImpl <serverName>\n");
	System.exit(1);
    }

    public static void main(String[] args) {
	
	if (args.length != 1) {
	    usage();
	}

	if (System.getSecurityManager() == null) {
	    System.setSecurityManager(new RMISecurityManager());
	}
	

	String serverName = args[0];

	ProcessorImpl processor = null;
	try {
	    System.out.print("Initializing object... ");
	    processor = new ProcessorImpl(serverName);
	    System.out.println("done!");
            Naming.rebind("//localhost/Processor", processor);
            System.out.println("Processor bound in registry");
	    System.out.print("Connecting server '" + serverName + "'... ");
	    Registry registry = (Registry)Naming.lookup("//" + serverName + "/Registry"); 
	    registry.register(processor.getHostAddress()); 
	    System.out.println("done!");
	    System.out.println("Ready to process.");
	} catch (Exception e) {
	    e.printStackTrace();
	    System.exit(1);
	}


    }


    class KillTask extends TimerTask {

	public KillTask() {
	    super();
	}

	public void run() {
	    System.out.println("Exiting...");
	    System.exit(0);
	}
    }

}
