/****************************************************************************
   File:        MainPanel.java
   Description: MainPanel that show user controls.
   Project:     MAC 322 - Distributed Scheduler
   Authors:     Nelio Pereira (nelio@ime.usp.br)
   Institution: Department of Computer Science
                University of Sao Paulo
******************************************************************************/

package sod.server;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.util.*;
import javax.swing.*;

import sod.util.FileClassLoader;

public class MainPanel extends JPanel {

    private Monitor monitor;

    private JList registeredHosts;
    private JTextArea logArea;
    private JFileChooser fileChooser;
    private JButton execClassButton;
    private JButton searchClassButton;
    private JButton unregisterButton;
    private JTextField classTextField;
    private Class abstractAppClass;

    public MainPanel(Monitor monitor) {
	super(true);
        this.monitor = monitor;

	try {
	    abstractAppClass = Class.forName("sod.util.AbstractApplication");
	} catch (ClassNotFoundException e) {
	    writeLog(e.getMessage());
	}

	setLayout(new BorderLayout());
	
	//fileChooser = new JFileChooser(new File("/home/nelio/Projects/Devel/Scheduler/build/classes/sod/exec"));
	fileChooser = new JFileChooser();
	ExampleFileFilter filter = new ExampleFileFilter("class");
	filter.setDescription("Java Class Files");
	fileChooser.setFileFilter(filter);
	fileChooser.setAcceptAllFileFilterUsed(false);
	fileChooser.setMultiSelectionEnabled(false);

	JPanel panelA = new JPanel();
	panelA.setBorder(BorderFactory.createEtchedBorder());
	panelA.setLayout(new BorderLayout());
	
	registeredHosts = new JList(monitor.getHosts());
	JScrollPane scroller2 = new JScrollPane(registeredHosts);
	scroller2.getViewport().setScrollMode(JViewport.BACKINGSTORE_SCROLL_MODE);
	panelA.add(new JLabel("Registered Hosts:"), BorderLayout.CENTER);
	panelA.add(scroller2, BorderLayout.SOUTH);
	unregisterButton = new JButton("Unregister");
	unregisterButton.addActionListener(new UnregisterButtonAction(this));
	panelA.add(unregisterButton, BorderLayout.EAST);
	add(panelA, BorderLayout.NORTH);

	JPanel panelD = new JPanel();
	panelD.setLayout(new BorderLayout());

	JPanel panelB = new JPanel();
	panelB.setBorder(BorderFactory.createEtchedBorder());
	panelB.setLayout(new BorderLayout());
	panelB.add(new JLabel("Create process:"), BorderLayout.NORTH);
	
	JPanel panel1 = new JPanel();
	panel1.setLayout(new BorderLayout());
	panel1.add(new JLabel("Class: "), BorderLayout.WEST);
	
	JPanel panel2 = new JPanel();
	panel2.setLayout(new BorderLayout());
	classTextField = new JTextField();
	//classTextField = new JTextField("C:\\Projects\\Devel\\Scheduler\\build\\classes\\sod\\exec\\Run1.class");
	//classTextField = new JTextField("/home/nelio/Projects/Devel/Scheduler/build/classes/sod/exec/Run1.class");
        classTextField.setBackground(Color.white);
        classTextField.setEditable(false);
	panel2.add(classTextField, BorderLayout.NORTH);
	searchClassButton = new JButton("Search file...");
	searchClassButton.addActionListener(new SearchClassButtonAction(this));
	panel2.add(searchClassButton, BorderLayout.SOUTH);
	panel1.add(panel2, BorderLayout.CENTER);
	
	execClassButton = new JButton("Execute");
        execClassButton.addActionListener(new ExecClassButtonAction(this));
	panel1.add(execClassButton, BorderLayout.EAST);
	
	panelB.add(panel1, BorderLayout.CENTER);
	panelD.add(panelB, BorderLayout.NORTH);

	JPanel panelC = new JPanel();
	panelC.setBorder(BorderFactory.createEtchedBorder());
	panelC.setLayout(new BorderLayout());
	panelC.add(new JLabel("Log messages:"), BorderLayout.NORTH);

	logArea = new JTextArea();
	logArea.setEditable(false);
	JScrollPane scroller = new JScrollPane();
	JViewport port2 = scroller.getViewport();
	port2.add(logArea);
	port2.setScrollMode(JViewport.BACKINGSTORE_SCROLL_MODE);
	panelC.add(scroller, BorderLayout.CENTER);
	panelD.add(panelC, BorderLayout.CENTER);

	add(panelD, BorderLayout.CENTER);
    }

    public Monitor getMonitor() {
	return monitor;
    }

    public JList getRegisteredList() {
	return registeredHosts;
    }

    public void buildHostList(String[] hosts) {
	registeredHosts.removeAll();
	registeredHosts.setListData(hosts);
	validateTree();
	repaint();
    }

    public void writeLog(String msg) {
	SimpleDateFormat dateFormat = new SimpleDateFormat("[dd-MM-yyyy hh:mm:ss z] - ");
	String dateString = dateFormat.format(new Date());
	logArea.append(dateString + msg + "\n");
	logArea.setCaretPosition(logArea.getText().length());
    }

    public void setClassName(String className) {
	classTextField.setText(className);
    }

    public String getClassName() {
	return classTextField.getText();
    }

    class UnregisterButtonAction extends AbstractAction {
	
	private MainPanel mainPanel;
	
	public UnregisterButtonAction(MainPanel mainPanel) {
	    this.mainPanel = mainPanel;
	}

	public void actionPerformed(ActionEvent e) {
	    JList registeredList = mainPanel.getRegisteredList();
	    String hostName = (String)registeredList.getSelectedValue();
	    if (hostName != null) {
		mainPanel.getMonitor().unregister(hostName);
	    }
	}
    }

    class ExecClassButtonAction extends AbstractAction {

	private MainPanel mainPanel;

	public ExecClassButtonAction(MainPanel mainPanel) {
	    this.mainPanel = mainPanel;
	}

        public void actionPerformed(ActionEvent e) {
	    if (mainPanel.getMonitor().getHosts().length == 0) {
		writeLog("No registered hosts!");
		return;
	    } 
	    
	    String classFileName = mainPanel.getClassName();
	    if ((classFileName != null) && (classFileName.length() > 0)) {
		File file = new File(classFileName);
		if (! file.isFile()) {
		    writeLog("'" + classFileName + "' is not a regular file.");
		    return;
		} 
		
		if (! classFileName.endsWith(".class")) {
		    writeLog("'" + classFileName + "' is not a valid file. Select only .class files.");
		    return;
		} 
		
		if (! file.canRead()) {
		    writeLog("Cannot read file '" + classFileName + "'.");
		    return;
		} 
		
		Class classRef = (new FileClassLoader(file)).loadClass();
		if (! classRef.getSuperclass().equals(abstractAppClass)) {
		    writeLog("File '" + classFileName + 
			     "' does not contain a valid application class. Use only subclasses of " + 
			     abstractAppClass.getName() + ".");
		    return;
		}

		writeLog("Executing file '" + classFileName + "'...");
		monitor.execute(file, classRef);
		buildHostList(monitor.getHosts());
		classTextField.setText("");

	    } else {
		writeLog("No file selected!");
	    }
	}
    }


    class SearchClassButtonAction extends AbstractAction {

	private MainPanel mainPanel;

	public SearchClassButtonAction(MainPanel mainPanel) {
	    this.mainPanel = mainPanel;
	}

        public void actionPerformed(ActionEvent e) {
	    int returnVal = fileChooser.showDialog(mainPanel, "Select File");
	    if (returnVal == JFileChooser.APPROVE_OPTION) {
		mainPanel.setClassName(fileChooser.getSelectedFile().getAbsolutePath());
	    }
	}
    }
	
}
