/****************************************************************************
   File:        Monitor.java
   Description: Monitor in the main server side class. It's responsible for registering processors, and for link the core classes to the GUI.
   Project:     MAC 322 - Distributed Scheduler
   Authors:     Nelio Pereira (nelio@ime.usp.br)
   Institution: Department of Computer Science
                University of Sao Paulo
******************************************************************************/

package sod.server;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.rmi.server.UnicastRemoteObject;

import sod.util.Process;
import sod.util.Processor;
import sod.util.SchedulerUI;
import sod.util.Registry;

/**
 *
 * GUI that monitors the scheduling system.
 */
public class Monitor extends UnicastRemoteObject implements Registry {

    private JPanel monitorPanel;
    
    private long EXIT_DELAY = 5000;

    /* Run the Load Adapter every 15 seconds */
    private int ADAPTER_SLEEP = 15;
    private Scheduler scheduler;
    private LoadAdapter loadAdapter;

    private MyMenuBar menuBar;
    private JPanel panel;
    private MainPanel mainPanel;
    private Hashtable procPanels;


    public Monitor() throws RemoteException {
	super();
	monitorPanel = new JPanel(true);
        scheduler = new Scheduler();
	loadAdapter = new LoadAdapter(this, scheduler, ADAPTER_SLEEP);
	loadAdapter.start();
        mainPanel = new MainPanel(this);
	procPanels = new Hashtable();

	try {
	    UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
	} catch (Exception e) {
	    System.err.println("Error loading L&F");
	    e.printStackTrace();
	}
	monitorPanel.setBorder(BorderFactory.createEtchedBorder());
	monitorPanel.setLayout(new BorderLayout());

        menuBar = new MyMenuBar(this);
	monitorPanel.add(menuBar, BorderLayout.NORTH);	

	changePanel(mainPanel);
    }


    public MyMenuBar getMenuBar() {
	return menuBar;
    }


    public void killProcessors() {
	Vector processors = scheduler.getProcessors();
	for (int i=0; i < processors.size(); i++) {
	    unregister((Processor)processors.get(i));
	}
    }


    /**
     *
     * Unregister a host from the Scheduler.
     */
    public void unregister(String hostName) {
	Processor processor = scheduler.getProcessor(hostName);
	unregister(processor);
    }

    /**
     *
     * Unregister a processor from the Scheduler.
     */
    public void unregister(Processor processor) {
	try {
	    processor.exit(EXIT_DELAY);
	    scheduler.delProcessor(processor.getHostName());
	    mainPanel.writeLog(processor.getHostName() + " unregistered.");
	} catch (RemoteException e) {
	    mainPanel.writeLog(e.getMessage());
	    e.printStackTrace();
	}
	mainPanel.buildHostList(scheduler.getHosts());
	if (panel instanceof ProcPanel) {
	    ((ProcPanel)panel).buildHostList(scheduler.getHosts());
	}	
	panel.validate();
    }


    /**
     *
     * Register a host from the Scheduler.
     * @param hostName The hostname where this processor is located.
     */
    public void register(String hostName) throws RemoteException {
	Processor processor = null;
	try {
	    processor = (Processor)Naming.lookup("//" + hostName + "/Processor"); 
	} catch (Exception e) {
	    mainPanel.writeLog(e.getMessage());
	    e.printStackTrace();
	}
	scheduler.addProcessor(processor);
	mainPanel.buildHostList(scheduler.getHosts());
	try {
	    mainPanel.writeLog(processor.getHostName() + " registered.");
	} catch (RemoteException e) {
	    mainPanel.writeLog(e.getMessage());
	    e.printStackTrace();
	}
	if (panel instanceof ProcPanel) {
	    ((ProcPanel)panel).buildHostList(scheduler.getHosts());
	}	
	panel.validate();
    }

    public JPanel getPanel() {
	return monitorPanel;
    }

    public java.util.List getProcesses() {
       return scheduler.getProcesses();
    }

    public String[] getHosts() {
	return scheduler.getHosts();
    }

    public MainPanel getMainPanel() {
	return mainPanel;
    }
    
    /**
     *
     * Returns the panel of a process.
     * @param pid The process ID.
     * @return ProcPanel This process's panel.
     */
    public ProcPanel getProcPanel(int pid) {
	return (ProcPanel)procPanels.get(new Integer(pid));
    }

    /**
     *
     * Execute a class.
     * @param file Class to be loaded.
     */
    public void execute(File file, Class classRef) {
       Process process = scheduler.createProcess(file, classRef);
       ProcPanel procPanel = new ProcPanel(this, process);
       try {
	   SchedulerUI schedulerUI = new SchedulerUIImpl(procPanel);
	   int pid = process.getPID();
	   Naming.rebind("//localhost/SchedulerUI" + pid, schedulerUI);
	   System.err.println("SchedulerUI" + pid + " bound in registry.");  
       } catch (Exception e) {
	   mainPanel.writeLog(e.getMessage());
	   e.printStackTrace();
       }
       scheduler.execute(process);
       procPanels.put(new Integer(process.getPID()), procPanel);
       menuBar.build();
    }

    public void migrateProcess(Process process, String hostName) {
	scheduler.migrateProcess(process, hostName);
	ProcPanel procPanel = getProcPanel(process.getPID());
	procPanel.setProcess(process);
	mainPanel.writeLog("Migrating " + process.getClassName() + " (PID=" + process.getPID() +") to " + hostName);
	menuBar.build();
    }

    /**
     *
     * Changes the visible panel.
     * @param newPanel The new panel to be shown.
     */
    public void changePanel(JPanel newPanel) {
	if (panel != null) {
	    monitorPanel.remove(panel);
	}
	panel = newPanel;
	if (panel instanceof ProcPanel) { 
	    ((ProcPanel)panel).buildHostList(scheduler.getHosts());
	} 
	monitorPanel.add(panel, BorderLayout.CENTER);
	monitorPanel.validate();
	panel.repaint();
    }


    public void delPanel(int pid) {
	procPanels.remove(new Integer(pid));
	menuBar.build();
    }
    

    /*************************************************/


    public static void main (String[] args) {
        JFrame frame = new JFrame();
	frame.setTitle("Distributed Scheduler 1.0");
	frame.setBackground(Color.lightGray);
	frame.getContentPane().setLayout(new BorderLayout());
	Monitor monitor = null;
	try {
	    monitor = new Monitor();
	} catch (RemoteException e) {
	    e.printStackTrace();
	    System.exit(1);
	}
        frame.getContentPane().add(monitor.getPanel(), BorderLayout.CENTER);
	frame.addWindowListener(new AppCloser(monitor));
	frame.pack();
	frame.setSize(550, 600);
	frame.setResizable(false);
        frame.show();


	// Create and install a security manager
	if (System.getSecurityManager() == null) {
	    System.setSecurityManager(new RMISecurityManager());
	}
	
	try {
	    Naming.rebind("//localhost/Registry", monitor);
            System.err.println("Monitor bound in registry.");
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    protected static final class AppCloser extends WindowAdapter {
	
	private Monitor monitor;

	public AppCloser(Monitor monitor) {
	    this.monitor = monitor;
	}

        public void windowClosing(WindowEvent e) {
	    monitor.killProcessors();
	    System.exit(0);
	}
    }

}



